@@ -7,7 +7,6 @@
 // except according to those terms.
 
 use std::cmp;
-use std::ptr;
 use std::slice;
 
 use imp_prelude::*;
@@ -84,13 +83,13 @@ impl<A, S, D> ArrayBase<S, D> where S: Data<Elem=A>, D: Dimension
     pub fn to_owned(&self) -> OwnedArray<A, D>
         where A: Clone
     {
-        let data = if let Some(slc) = self.as_slice() {
-            slc.to_vec()
+        let (data, strides) = if let Some(slc) = self.as_slice_memory_order() {
+            (slc.to_vec(), self.strides.clone())
         } else {
-            self.iter().cloned().collect()
+            (self.iter().cloned().collect(), self.dim.default_strides())
         };
         unsafe {
-            ArrayBase::from_vec_dim_unchecked(self.dim.clone(), data)
+            ArrayBase::from_vec_dim_stride_unchecked(self.dim.clone(), strides, data)
         }
     }
 
@@ -551,6 +550,7 @@ impl<A, S, D> ArrayBase<S, D> where S: Data<Elem=A>, D: Dimension
         if self.strides == defaults {
             return true;
         }
+        if self.ndim() == 1 { return false; }
         // check all dimensions -- a dimension of length 1 can have unequal strides
         for (&dim, (&s, &ds)) in zipsl(self.dim.slice(),
                                        zipsl(self.strides(), defaults.slice()))
@@ -562,8 +562,56 @@ impl<A, S, D> ArrayBase<S, D> where S: Data<Elem=A>, D: Dimension
         true
     }
 
-    /// Return the array’s data as a slice, if it is contiguous and
-    /// the element order corresponds to the memory order. Return `None` otherwise.
+    fn is_contiguous(&self) -> bool {
+        let defaults = self.dim.default_strides();
+        if self.strides == defaults {
+            return true;
+        }
+        if self.ndim() == 1 { return false; }
+        let order = self.strides._fastest_varying_stride_order();
+        let strides = self.strides.slice();
+
+        // FIXME: Negative strides
+        let dim = self.dim.slice();
+        let mut cstride = 1;
+        for &i in order.slice() {
+            // a dimension of length 1 can have unequal strides
+            if dim[i] != 1 && strides[i] != cstride {
+                return false;
+            }
+            cstride *= dim[i];
+        }
+        true
+    }
+
+    /// Return a pointer to the first element in the array.
+    ///
+    /// Raw access to array elements needs to follow the strided indexing
+    /// scheme: an element at multi-index *I* in an array with strides *S* is 
+    /// located at offset
+    ///
+    /// *Σ<sub>0 ≤ k < d</sub> I<sub>k</sub> × S<sub>k</sub>*
+    ///
+    /// where *d* is `self.ndim()`.
+    #[inline(always)]
+    pub fn as_ptr(&self) -> *const A {
+        self.ptr
+    }
+
+    /// Return a mutable pointer to the first element in the array.
+    #[inline(always)]
+    pub fn as_mut_ptr(&mut self) -> *mut A
+        where S: DataMut
+    {
+        self.ensure_unique(); // for RcArray
+        self.ptr
+    }
+
+    /// Return the array’s data as a slice, if it is contiguous and in standard order.
+    /// Return `None` otherwise.
+    ///
+    /// If this function returns `Some(_)`, then the element order in the slice
+    /// corresponds to the logical order of the array’s elements.
     pub fn as_slice(&self) -> Option<&[A]> {
         if self.is_standard_layout() {
             unsafe {
@@ -574,8 +622,8 @@ impl<A, S, D> ArrayBase<S, D> where S: Data<Elem=A>, D: Dimension
         }
     }
 
-    /// Return the array’s data as a slice, if it is contiguous and
-    /// the element order corresponds to the memory order. Return `None` otherwise.
+    /// Return the array’s data as a slice, if it is contiguous and in standard order.
+    /// Return `None` otherwise.
     pub fn as_slice_mut(&mut self) -> Option<&mut [A]>
         where S: DataMut
     {
@@ -589,6 +637,38 @@ impl<A, S, D> ArrayBase<S, D> where S: Data<Elem=A>, D: Dimension
         }
     }
 
+    /// Return the array’s data as a slice if it is contiguous,
+    /// return `None` otherwise.
+    ///
+    /// If this function returns `Some(_)`, then the elements in the slice
+    /// have whatever order the elements have in memory.
+    ///
+    /// Implementation notes: Does not yet support negatively strided arrays.
+    pub fn as_slice_memory_order(&self) -> Option<&[A]> {
+        if self.is_contiguous() {
+            unsafe {
+                Some(slice::from_raw_parts(self.ptr, self.len()))
+            }
+        } else {
+            None
+        }
+    }
+
+    /// Return the array’s data as a slice if it is contiguous,
+    /// return `None` otherwise.
+    pub fn as_slice_memory_order_mut(&mut self) -> Option<&mut [A]>
+        where S: DataMut
+    {
+        if self.is_contiguous() {
+            self.ensure_unique();
+            unsafe {
+                Some(slice::from_raw_parts_mut(self.ptr, self.len()))
+            }
+        } else {
+            None
+        }
+    }
+
     /// Transform the array into `shape`; any shape with the same number of
     /// elements is accepted.
     ///
@@ -808,6 +888,7 @@ impl<A, S, D> ArrayBase<S, D> where S: Data<Elem=A>, D: Dimension
     /// **Note:** Data memory order may not correspond to the index order
     /// of the array. Neither is the raw data slice is restricted to just the
     /// array’s view.<br>
+    #[cfg_attr(has_deprecated, deprecated(note="Use .as_slice_memory_order() instead"))]
     pub fn raw_data(&self) -> &[A]
         where S: DataOwned,
     {
@@ -822,6 +903,7 @@ impl<A, S, D> ArrayBase<S, D> where S: Data<Elem=A>, D: Dimension
     ///
     /// **Note:** The data is uniquely held and nonaliased
     /// while it is mutably borrowed.
+    #[cfg_attr(has_deprecated, deprecated(note="Use .as_slice_memory_order_mut() instead"))]
     pub fn raw_data_mut(&mut self) -> &mut [A]
         where S: DataOwned + DataMut,
     {
@@ -925,7 +1007,6 @@ impl<A, S, D> ArrayBase<S, D> where S: Data<Elem=A>, D: Dimension
         self.unordered_foreach_mut(move |elt| f(elt, rhs_elem));
     }
 
-    // FIXME: Guarantee the order here or not?
     /// Traverse two arrays in unspecified order, in lock step,
     /// calling the closure `f` on each element pair.
     ///
@@ -959,8 +1040,9 @@ impl<A, S, D> ArrayBase<S, D> where S: Data<Elem=A>, D: Dimension
         where F: FnMut(B, &'a A) -> B, A: 'a
     {
         if let Some(slc) = self.as_slice() {
-            for elt in slc {
-                init = f(init, elt);
+            // FIXME: Use for loop when slice iterator is perf is restored
+            for i in 0..slc.len() {
+                init = f(init, &slc[i]);
             }
             return init;
         }
@@ -988,27 +1070,21 @@ impl<A, S, D> ArrayBase<S, D> where S: Data<Elem=A>, D: Dimension
     ///               [false, true]])
     /// );
     /// ```
-    pub fn map<'a, B, F>(&'a self, mut f: F) -> OwnedArray<B, D>
+    pub fn map<'a, B, F>(&'a self, f: F) -> OwnedArray<B, D>
         where F: FnMut(&'a A) -> B,
               A: 'a,
     {
-        // Use an `unsafe` block to do this efficiently.
-        // We know that iter will produce exactly .size() elements,
-        // and the loop can vectorize if it's clean (without branch
-        // to grow the vector).
-        let mut res = Vec::with_capacity(self.dim.size());
-        let mut out_ptr = res.as_mut_ptr();
-        let mut len = 0;
-        for elt in self.iter() {
+        if let Some(slc) = self.as_slice_memory_order() {
+            let v = ::iterators::to_vec(slc.iter().map(f));
             unsafe {
-                ptr::write(out_ptr, f(elt));
-                len += 1;
-                res.set_len(len);
-                out_ptr = out_ptr.offset(1);
+                ArrayBase::from_vec_dim_stride_unchecked(
+                    self.dim.clone(), self.strides.clone(), v)
+            }
+        } else {
+            let v = ::iterators::to_vec(self.iter().map(f));
+            unsafe {
+                ArrayBase::from_vec_dim_unchecked(self.dim.clone(), v)
             }
-        }
-        unsafe {
-            ArrayBase::from_vec_dim_unchecked(self.dim.clone(), res)
         }
     }
 }
