@@ -47,7 +47,7 @@
 //!   + `.fold()` and `.zip_mut_with()` are the most efficient ways to
 //!     perform single traversal and lock step traversal respectively.
 //!   + `.iter()` and `.iter_mut()` are efficient for contiguous arrays.
-//! - There is experimental bridging to the linear algebra package `rblas`.
+//!   + Can use BLAS in some operations (`dot` and `mat_mul`).
 //!
 //! ## Crate Feature Flags
 //!
@@ -58,10 +58,14 @@
 //!   - Requires Rust 1.8, will be default soon.
 //!   - Enables the compound assignment operators
 //! - `rustc-serialize`
-//!   - Optional, stable
+//!   - Optional, compatible with Rust stable
 //!   - Enables serialization support
+//! - `blas`
+//!   - Optional and experimental, compatible with Rust stable
+//!   - Enable transparent BLAS support for matrix multiplication. Pluggable
+//!     backend via `blas-sys`.
 //! - `rblas`
-//!   - Optional, stable
+//!   - ***Deprecated:*** replaced by separate crate `ndarray-rblas`
 //!   - Enables `rblas` integration
 //!
 #![cfg_attr(all(feature = "assign_ops", not(has_assign)),
@@ -74,6 +78,8 @@ extern crate rustc_serialize as serialize;
 
 #[cfg(feature = "rblas")]
 extern crate rblas;
+#[cfg(feature="blas")]
+extern crate blas_sys;
 
 extern crate itertools;
 extern crate num as libnum;
@@ -235,11 +241,14 @@ pub type Ixs = isize;
 /// for element indices in `.get()` and `array[index]`. The dimension type `Vec<Ix>`
 /// allows a dynamic number of axes.
 ///
-/// The default memory order of an array is *row major* order, where each
-/// row is contiguous in memory.
-/// A *column major* (a.k.a. fortran) memory order array has
+/// The default memory order of an array is *row major* order (a.k.a ”c” order),
+/// where each row is contiguous in memory.
+/// A *column major* (a.k.a. “f” or fortran) memory order array has
 /// columns (or, in general, the outermost axis) with contiguous elements.
 ///
+/// The logical order of any array’s elements is the row major order.
+/// The iterators `.iter(), .iter_mut()` always adhere to this order, for example.
+///
 /// ## Slicing
 ///
 /// You can use slicing to create a view of a subset of the data in
@@ -504,13 +513,21 @@ impl<A, S, D> ArrayBase<S, D>
         where S: DataMut,
               F: FnMut(&mut A)
     {
-        if let Some(slc) = self.as_slice_mut() {
-            for elt in slc {
-                f(elt);
+        if let Some(slc) = self.as_slice_memory_order_mut() {
+            // FIXME: Use for loop when slice iterator is perf is restored
+            for i in 0..slc.len() {
+                f(&mut slc[i]);
             }
             return;
         }
-        for row in self.inner_iter_mut() {
+        for mut row in self.inner_iter_mut() {
+            if let Some(slc) = row.as_slice_mut() {
+                // FIXME: Use for loop when slice iterator is perf is restored
+                for i in 0..slc.len() {
+                    f(&mut slc[i]);
+                }
+                continue;
+            }
             for elt in row {
                 f(elt);
             }
